//+------------------------------------------------------------------+
//|                                                  Heiken Ashi.mq4 |
//|                      Copyright c 2004, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
//| For Heiken Ashi we recommend next chart settings ( press F8 or   |
//| select on menu 'Charts'->'Properties...'):                       |
//|  - On 'Color' Tab select 'Black' for 'Line Graph'                |
//|  - On 'Common' Tab disable 'Chart on Foreground' checkbox and    |
//|    select 'Line Chart' radiobutton                               |
//+------------------------------------------------------------------+
#property copyright "Copyright  2004, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  Blue
#property indicator_width1  2
#property indicator_color2  Red
#property indicator_width2  2
#property indicator_maximum 1.05
#property indicator_minimum -0.05

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double Up[];
double Dn[];
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
  {
   IndicatorBuffers(7);
   SetIndexBuffer(0, Up); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, Dn); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2, ExtMapBuffer1); 
   SetIndexBuffer(3, ExtMapBuffer2);
   SetIndexBuffer(4, ExtMapBuffer3); 
   SetIndexBuffer(5, ExtMapBuffer4); 
   SetIndexBuffer(6, trend);
      
     //
     //
     //
     //
     //
      
     indicatorFileName = WindowExpertName();
     calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
     returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
     timeFrame         = stringToTimeFrame(TimeFrame);
      
     //
     //
     //
     //
     //
   
   IndicatorShortName(timeFrameToString(timeFrame)+  "  Heiken Ashi");
 return(0);
}

//
//
//
//
//

int deinit(){  return(0); }

//
//
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   int counted_bars=IndicatorCounted();
   int pos,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit = MathMin(Bars - counted_bars,Bars-1);
           if (returnBars)  { Up[0] = limit+1; return(0); }
   
   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {

          for(pos=limit; pos >= 0; pos--)
          {
          
          double haClose = (Open[pos]+High[pos]  + Low[pos]+Close[pos])/4;
          double haOpen  = (ExtMapBuffer3[pos+1] + ExtMapBuffer4[pos+1])/2;
          double haHigh  = MathMax(High[pos], MathMax(haOpen,haClose));
          double haLow   = MathMin(Low[pos],  MathMin(haOpen,haClose));
          
          if (haOpen < haClose) 
          { 
          ExtMapBuffer1[pos] = haLow;  
          ExtMapBuffer2[pos] = haHigh; 
          } 
          else              
          { 
          ExtMapBuffer1[pos] = haHigh; 
          ExtMapBuffer2[pos] = haLow;  
          } 
          ExtMapBuffer3[pos] = haOpen;
          ExtMapBuffer4[pos] = haClose;      
          
          Up[pos]    = EMPTY_VALUE; 
          Dn[pos]    = EMPTY_VALUE;
          trend[pos] = trend[pos+1];
          
          if (ExtMapBuffer3[pos] < ExtMapBuffer4[pos]) trend[pos] =  1;
          if (ExtMapBuffer3[pos] > ExtMapBuffer4[pos]) trend[pos] = -1;
          
          if (trend[pos] ==  1) Up[pos] = 1;
          if (trend[pos] == -1) Dn[pos] = 1;
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(pos = limit; pos >= 0; pos--)
   {
      int y = iBarShift(NULL,timeFrame,Time[pos]);
         Up[pos]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",0,y);
         Dn[pos]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",1,y);
         trend[pos]= iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",6,y);
   }
   manageAlerts();
   return(0);         
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Heiken Ashi  ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Heiken Ashi "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//
//
//
//
//



          






